/* $Id: pysaviElements.c,v 1.1.1.1 2004/11/02 03:57:58 mjp Exp $ */
#include <Python.h>
#include <unistd.h>
#include "csavi3c.h"
#include "pysaviUtil.h"
#include "pysaviTypes.h"

/* pysavi Elements
 *
 * (CIEnum* ........... PyS_GenericEnum; pysaviEnum.c)
 * 
 * CISweepResults...... PySSweepResults
 * CIIDEDetails........ PySIDEDetails
 * CIEngineConfig...... PySEngineConfig
 * CISweepError........ PySSweepError
 * CIVersionChecksum... PySVersionChecksum
 *
 * These objects are returned by the iternext() functions of their respective
 * PySEnum* creators, or sometimes instantiated for pysavi callback handlers.
 *
 * N.B.: dealloc routines do _not_ Release() the underlying interface objects
 * -- this is the responsibility of the parent (or the hidden mechanics of
 * SAVI callbacks).
 * 
 */

/* === Objects and Types =================================================== */

typedef struct {
  PyObject_HEAD
  CISweepUnknown2 *element;
} PySGenericElement;

typedef PySGenericElement PySSweepResults;
typedef PySGenericElement PySIDEDetails;
typedef PySGenericElement PySEngineConfig;
typedef PySGenericElement PySSweepError;
typedef PySGenericElement PySVersionChecksum;

/* shared methods */

static void
PySGenericElement_dealloc(PySGenericElement *self) {
  pysIThingSafeRelease(self->element);
  PyMem_Del((PyObject *)self);
}

/* SweepResults */

/* PySSweepResults_converter
 *
 * PyArg_Parse converter (for Disinfect) to translate PySSweepResults objects
 * to their CISweepResults member.
 */
int
PySSweepResults_converter(PyObject *o, void **out) {
  PySSweepResults *sr = NULL;

  if (!PyObject_TypeCheck(o, &PySSweepResults_Type)) {
    PyErr_SetString(PyExc_TypeError, "function expects SweepResults object");
    return 0;
  }
  sr = (PySSweepResults *)o;
  *out = (void *)(sr->element);
  return 1;
}

static char SR_doc[] = 
"Represents a virus found; created by the iterator returned by any of the\n"
"Sweep... or Disinfect... methods.  Read-only attributes:\n"
"\n"
"  IsDisinfectable - whether or not virus is disinfectable\n"
"  LocationInformation - file or subfile in which virus was detected\n"
"  VirusName - the name of the virus found\n"
"  VirusType - integral type code (not often used)\n";

PyObject *
PySSweepResults_New(CISweepResults *p) {
  PySGenericElement *o;
  o = PyObject_New(PySSweepResults, &PySSweepResults_Type);
  if (o)
    o->element = (CISweepUnknown2 *)p;
  return (PyObject *)o;
}

static PyObject *
SR_GetVirusName(PySGenericElement *self, PyObject *args) {
  CISweepResults *sr = (CISweepResults *)self->element;
  return pysIThingGetString(sr, sr->pVtbl->GetVirusName);
}

static PyObject *
SR_GetLocationInformation(PySGenericElement *self, PyObject *args) {
  CISweepResults *sr = (CISweepResults *)self->element;
  return pysIThingGetString(sr, sr->pVtbl->GetLocationInformation);
}

static PyObject *
SR_IsDisinfectable(PySGenericElement *self, PyObject *args) {
  CISweepResults *sr = (CISweepResults *)self->element;
  return pysIThingGetInt(sr, sr->pVtbl->IsDisinfectable);
}

static PyObject *
SR_GetVirusType(PySGenericElement *self, PyObject *args) {
  CISweepResults *sr = (CISweepResults *)self->element;
  return pysIThingGetInt(sr, sr->pVtbl->GetVirusType);
}

static PyGetSetDef SR_getsets[] = {
  {"VirusName", (getter)SR_GetVirusName, NULL, "Name of the virus", NULL},
  {"LocationInformation", (getter)SR_GetLocationInformation, NULL, "Location of virus", NULL},
  {"IsDisinfectable", (getter)SR_IsDisinfectable, NULL, "Is virus disinfectable", NULL},
  {"VirusType", (getter)SR_GetVirusType, NULL, "Type of virus", NULL},
  {0}
};

PyTypeObject PySSweepResults_Type = {
  /* The ob_type field must be initialized in the module init function
   * to be portable to Windows without using C++. */
  PyObject_HEAD_INIT(NULL)
  0,          /*ob_size*/
  "_savi.SweepResults",     /*tp_name*/
  sizeof(PySGenericElement),   /*tp_basicsize*/
  0,          /*tp_elementsize*/
  /* methods */
  (destructor)PySGenericElement_dealloc, /*tp_dealloc*/
  0,          /*tp_print*/
  0,          /*tp_getattr*/
  0,          /*tp_setattr*/
  0,          /*tp_compare*/
  0,          /*tp_repr*/
  0,          /*tp_as_number*/
  0,          /*tp_as_sequence*/
  0,          /*tp_as_mapping*/
  0,          /*tp_hash*/
  0,          /*tp_call*/
  0,          /*tp_str*/
  PyObject_GenericGetAttr,          /*tp_getattro*/
  0,          /*tp_setattro*/
  0,          /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT,          /*tp_flags*/
  SR_doc,     /*tp_doc*/
  0,          /*tp_traverse*/
  0,          /*tp_clear*/
  0,          /*tp_richcompare*/
  0,          /*tp_weaklistoffset*/
  0,          /*tp_iter*/
  0,          /*tp_iternext*/
  0,          /*tp_methods*/
  0,          /*tp_members*/
  SR_getsets, /*tp_getset*/
  0,          /*tp_base*/
};

/* EngineConfig */

static char EC_doc[] =
"Represents a single engine configuration option; created by the iterator\n"
"returned from GetEngineConfig.  Read-only attributes:\n"
"\n"
"  Name - name of configuration option\n"
"  Type - integral type code (SOPHOS_TYPE_U16, e.g.)\n";

PyObject *
PySEngineConfig_New(CIEngineConfig *pConfig) {
  PySGenericElement *o;
  o = PyObject_New(PySEngineConfig, &PySEngineConfig_Type);
  if (o)
    o->element = (CISweepUnknown2 *)pConfig;
  return (PyObject *)o;
}

static PyObject *
EC_GetName(PySGenericElement *self, PyObject *args) {
  CIEngineConfig *ec = (CIEngineConfig *)self->element;
  return pysIThingGetString(ec, ec->pVtbl->GetName);
}

static PyObject *
EC_GetType(PySEngineConfig *self, PyObject *args) {
  CIEngineConfig *ec = (CIEngineConfig *)self->element;
  return pysIThingGetInt(ec, ec->pVtbl->GetType);
}

static PyGetSetDef EC_getsets[] = {
  {"Name", (getter)EC_GetName, NULL, "Name of engine configuration option", NULL},
  {"Type", (getter)EC_GetType, NULL, "Type of engine configuration option", NULL},
  {0}
};

PyTypeObject PySEngineConfig_Type = {
  /* The ob_type field must be initialized in the module init function
   * to be portable to Windows without using C++. */
  PyObject_HEAD_INIT(NULL)
  0,          /*ob_size*/
  "_savi.EngineConfig",     /*tp_name*/
  sizeof(PySGenericElement),   /*tp_basicsize*/
  0,          /*tp_elementsize*/
  /* methods */
  (destructor)PySGenericElement_dealloc, /*tp_dealloc*/
  0,          /*tp_print*/
  0, /*tp_getattr*/
  0,          /*tp_setattr*/
  0,          /*tp_compare*/
  0,          /*tp_repr*/
  0,          /*tp_as_number*/
  0,          /*tp_as_sequence*/
  0,          /*tp_as_mapping*/
  0,          /*tp_hash*/
  0,          /*tp_call*/
  0,          /*tp_str*/
  PyObject_GenericGetAttr,          /*tp_getattro*/
  0,          /*tp_setattro*/
  0,          /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT,          /*tp_flags*/
  EC_doc,     /*tp_doc*/
  0,          /*tp_traverse*/
  0,          /*tp_clear*/
  0,          /*tp_richcompare*/
  0,          /*tp_weaklistoffset*/
  0,          /*tp_iter*/
  0,          /*tp_iternext*/
  0,          /*tp_methods*/
  0,          /*tp_members*/
  EC_getsets, /*tp_getset*/
  0,          /*tp_base*/
};

/* IDEDetails */

static char ID_doc[] =
"Represents an update file in use by the engine; created by the iterator\n"
"returned by GetVirusEngineVersion.  Read-only attributes:\n"
"\n"
"  Name - name of update file\n"
"  State - integral state code (SOPHOS_IDE_VDL_FAILED, e.g.)\n"
"  Type - integral type code (SOPHOS_TYPE_VDL, e.g.)\n";


PyObject *
PySIDEDetails_New(CIIDEDetails *pDetails) {
  PySGenericElement *o;
  o = PyObject_New(PySIDEDetails, &PySIDEDetails_Type);
  if (o)
    o->element = (CISweepUnknown2 *)pDetails;
  return (PyObject *)o;
}

static PyObject *
ID_GetName(PySGenericElement *self, PyObject *args) {
  CIIDEDetails *id = (CIIDEDetails *)self->element;
  return pysIThingGetString(id, id->pVtbl->GetName);
}

static PyObject *
ID_GetState(PySGenericElement *self, PyObject *args) {
  CIIDEDetails *id = (CIIDEDetails *)self->element;
  return pysIThingGetInt(id, id->pVtbl->GetState);
}

static PyObject *
ID_GetType(PySGenericElement *self, PyObject *args) {
  CIIDEDetails *id = (CIIDEDetails *)self->element;
  return pysIThingGetInt(id, id->pVtbl->GetType);
}

static PyGetSetDef ID_getsets[] = {
  {"Name", (getter)ID_GetName, NULL, "IDE name", NULL},
  {"State", (getter)ID_GetState, NULL, "IDE integral state code", NULL},
  {"Type", (getter)ID_GetType, NULL, "IDE integral type code", NULL},
  {0}
};

PyTypeObject PySIDEDetails_Type = {
  /* The ob_type field must be initialized in the module init function
   * to be portable to Windows without using C++. */
  PyObject_HEAD_INIT(NULL)
  0,          /*ob_size*/
  "_savi.IDEDetails",     /*tp_name*/
  sizeof(PySIDEDetails),   /*tp_basicsize*/
  0,          /*tp_elementsize*/
  /* methods */
  (destructor)PySGenericElement_dealloc, /*tp_dealloc*/
  0,          /*tp_print*/
  0, /*tp_getattr*/
  0,          /*tp_setattr*/
  0,          /*tp_compare*/
  0,          /*tp_repr*/
  0,          /*tp_as_number*/
  0,          /*tp_as_sequence*/
  0,          /*tp_as_mapping*/
  0,          /*tp_hash*/
  0,          /*tp_call*/
  0,          /*tp_str*/
  PyObject_GenericGetAttr,          /*tp_getattro*/
  0,          /*tp_setattro*/
  0,          /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT,          /*tp_flags*/
  ID_doc,     /*tp_doc*/
  0,          /*tp_traverse*/
  0,          /*tp_clear*/
  0,          /*tp_richcompare*/
  0,          /*tp_weaklistoffset*/
  0,          /*tp_iter*/
  0, /*tp_iternext*/
  0,            /*tp_methods*/
  0,          /*tp_members*/
  ID_getsets, /*tp_getset*/
  0,          /*tp_base*/
};

/* SweepError */

static char SE_doc[] =
"Represents an error encountered during scanning; passed as an argument to\n"
"OnErrorFound callback methods.  Read-only attributes:\n"
"\n"
"  ErrorCode - integral error code (SOPHOS_SAVI2_ERROR_CORRUPT, e.g.)\n"
"  LocationInformation - file or subfile wherein error was encountered\n";

PyObject *
PySSweepError_New(CISweepError *pError) {
  PySGenericElement *o;
  o = PyObject_New(PySSweepError, &PySSweepError_Type);
  if (o)
    o->element = (CISweepUnknown2 *)pError;
  return (PyObject *)o;
}

static PyObject *
SE_GetLocationInformation(PySGenericElement *self, PyObject *args) {
  CISweepError *se = (CISweepError *)self->element;
  return pysIThingGetString(se, se->pVtbl->GetLocationInformation);
}
static PyObject *
SE_GetErrorCode(PySEngineConfig *self, PyObject *args) {
  HRESULT hr, ret;
  CISweepError *se = (CISweepError *)self->element;

  hr = se->pVtbl->GetErrorCode(se, &ret);
  if (!SOPHOS_SUCCEEDED(hr))
    return pysRaiseError(hr);

  return PyInt_FromLong((long) ret);
}

static PyGetSetDef SE_getsets[] = {
  {"LocationInformation", (getter)SE_GetLocationInformation, NULL,
    "Location of error", NULL},
  {"ErrorCode", (getter)SE_GetErrorCode, NULL,
    "Error code", NULL},
  {0}
};

PyTypeObject PySSweepError_Type = {
  /* The ob_type field must be initialized in the module init function
   * to be portable to Windows without using C++. */
  PyObject_HEAD_INIT(NULL)
  0,          /*ob_size*/
  "_savi.SweepError",     /*tp_name*/
  sizeof(PySSweepError),   /*tp_basicsize*/
  0,          /*tp_elementsize*/
  /* methods */
  (destructor)PySGenericElement_dealloc, /*tp_dealloc*/
  0,          /*tp_print*/
  0, /*tp_getattr*/
  0,          /*tp_setattr*/
  0,          /*tp_compare*/
  0,          /*tp_repr*/
  0,          /*tp_as_number*/
  0,          /*tp_as_sequence*/
  0,          /*tp_as_mapping*/
  0,          /*tp_hash*/
  0,          /*tp_call*/
  0,          /*tp_str*/
  PyObject_GenericGetAttr,          /*tp_getattro*/
  0,          /*tp_setattro*/
  0,          /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT,          /*tp_flags*/
  SE_doc,     /*tp_doc*/
  0,          /*tp_traverse*/
  0,          /*tp_clear*/
  0,          /*tp_richcompare*/
  0,          /*tp_weaklistoffset*/
  0,          /*tp_iter*/
  0, /*tp_iternext*/
  0,            /*tp_methods*/
  0,          /*tp_members*/
  SE_getsets, /*tp_getset*/
  0,          /*tp_base*/
};

/* VersionChecksum */

static char VC_doc[] =
"Represents a checksum of one component underlying the SAVI engine instance;\n"
"created by the iterator returned from GetVirusEngineVersion.  Read-only\n"
"attributes:\n"
"\n"
"  Type - integral type code (SOPHOS_TYPE_VDATA, e.g.)\n"
"  Value - checksum value\n";

PyObject *
PySVersionChecksum_New(CIVersionChecksum *pVersionCsum) {
  PySGenericElement *o;
  o = PyObject_New(PySVersionChecksum, &PySVersionChecksum_Type);
  if (o)
    o->element = (CISweepUnknown2 *)pVersionCsum;
  return (PyObject *)o;
}

static PyObject *
VC_GetType(PySGenericElement *self, PyObject *args) {
  CIVersionChecksum *se = (CIVersionChecksum *)self->element;
  return pysIThingGetInt(se, se->pVtbl->GetType);
}

static PyObject *
VC_GetValue(PySGenericElement *self, PyObject *args) {
  CIVersionChecksum *se = (CIVersionChecksum *)self->element;
  return pysIThingGetInt(se, se->pVtbl->GetValue);
}

static PyGetSetDef VC_getsets[] = {
  {"Type",  (getter)VC_GetType,  NULL, "Integral type of checksum", NULL},
  {"Value", (getter)VC_GetValue, NULL, "Value of checksum", NULL},
  {0}
};

PyTypeObject PySVersionChecksum_Type = {
  /* The ob_type field must be initialized in the module init function
   * to be portable to Windows without using C++. */
  PyObject_HEAD_INIT(NULL)
  0,          /*ob_size*/
  "_savi.VersionChecksum",     /*tp_name*/
  sizeof(PySVersionChecksum),   /*tp_basicsize*/
  0,          /*tp_elementsize*/
  /* methods */
  (destructor)PySGenericElement_dealloc, /*tp_dealloc*/
  0,          /*tp_print*/
  0, /*tp_getattr*/
  0,          /*tp_setattr*/
  0,          /*tp_compare*/
  0,          /*tp_repr*/
  0,          /*tp_as_number*/
  0,          /*tp_as_sequence*/
  0,          /*tp_as_mapping*/
  0,          /*tp_hash*/
  0,          /*tp_call*/
  0,          /*tp_str*/
  PyObject_GenericGetAttr,          /*tp_getattro*/
  0,          /*tp_setattro*/
  0,          /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT,          /*tp_flags*/
  VC_doc,     /*tp_doc*/
  0,          /*tp_traverse*/
  0,          /*tp_clear*/
  0,          /*tp_richcompare*/
  0,          /*tp_weaklistoffset*/
  0,          /*tp_iter*/
  0, /*tp_iternext*/
  0,            /*tp_methods*/
  0,          /*tp_members*/
  VC_getsets, /*tp_getset*/
  0,          /*tp_base*/
};

/* vim:et:ts=2:noai:
 *
 */
