/* $Id: pysaviEnum.c,v 1.1.1.1 2004/11/02 03:57:58 mjp Exp $ */
#include <Python.h>
#include <unistd.h>
#include "csavi3c.h"
#include "pysavi.h"
#include "pysaviTypes.h"
#include "pysaviUtil.h"

/* pysaviEnum
 *
 * Enumerators offer the following:
 *    typeCode
 *    __bool__()
 *    Reset()
 *    __iter__(), next()  (iterator protocol)
 */

typedef PyObject *(*PYSITERFUNC)(CISweepUnknown2 *);

static struct { REFIID riid; PYSITERFUNC f; } iter_tbl[] = {
  {&SOPHOS_IID_ENUM_SWEEPRESULTS, (PYSITERFUNC)PySSweepResults_New, },
  {&SOPHOS_IID_ENUM_ENGINECONFIG, (PYSITERFUNC)PySEngineConfig_New, },
  {&SOPHOS_IID_ENUM_IDEDETAILS,   (PYSITERFUNC)PySIDEDetails_New, },
  {&SOPHOS_IID_ENUM_CHECKSUM,     (PYSITERFUNC)PySVersionChecksum_New, },
};

typedef struct {
  PyObject_HEAD
  CIEnumGeneric *enumerator;
  PYSITERFUNC iternext;
  int nonzero;
} PySEnumGeneric;

PyTypeObject PySEnumGeneric_Type;

/* PySEnumGeneric_New
 *
 * Wrap a CIEnum* in a _savi.EnumGeneric (python iterator) object.
 * 
 * args:    REFIID ........... type of Enumerator interface passed
 *          CIEnumGeneric * .. actual Enumerator interface
 *          nonzero .......... boolean state of new iterator:
 *                              0  False
 *                             >0  True
 *                             <0  Undefined.  We could use Next()/Reset()
 *                                 to figure out our boolean state, but in
 *                                 practice this isn't necessary.
 * rtrn:    new PySEnumGeneric_Type object, NULL on exception
 * s-fx:    Resets() passed Enumerator
 */
PyObject *
PySEnumGeneric_New(REFIID riid, CIEnumGeneric *pEnum, int nonzero) {
  PySEnumGeneric *eg;
  int i;

  if (!riid || !pEnum || nonzero < 0) {
    PyErr_SetString(PyExc_RuntimeError,
                    "Invalid pysavi.EnumGeneric initialization");
    return NULL;
  }

  eg = PyObject_New(PySEnumGeneric, &PySEnumGeneric_Type);
  if (! eg) return NULL;

  eg->enumerator = pEnum;
  pysIEnumReset(pEnum);
  
  /* next() */
  eg->iternext = NULL;
  for (i = 0; i < sizeof(iter_tbl)/sizeof(iter_tbl[0]); i++) {
    if (IsEqualIID(riid, iter_tbl[i].riid)) {
      eg->iternext = iter_tbl[i].f;
    }
  }
  if (!eg->iternext) {
    PyMem_Del(eg);
    return pysRaiseError(SOPHOS_E_NOINTERFACE);
  }

  /*
   * In practice, only SOPHOS_IID_ENUM_SWEEPRESULTS will ever be empty, and
   * this is known in advance to the caller.
   */
  eg->nonzero = nonzero ? 1 : 0;

  return (PyObject *)eg;
}

static void
EG_dealloc(PySEnumGeneric *self) {
  pysIThingSafeRelease(self->enumerator);
  PyMem_Del((PyObject *)self);
}

static PyObject *
EG_iter(PySEnumGeneric *self) {
  Py_INCREF((PyObject *)self);
  return (PyObject *)self;
}

static PyObject *
EG_iternext(PySEnumGeneric *self) {
  CISweepUnknown2 *whatever = NULL;
  if (self->enumerator == NULL) {
    PyErr_SetString(PyExc_RuntimeError,
                    "Attempted iteration of uninitialized EnumGeneric");
    return NULL;
  }
  if (SOPHOS_S_OK == pysIEnumNext(self->enumerator, whatever)) {
    return self->iternext(whatever);
  }
  pysIThingSafeRelease(whatever); /* whatever is in uncertain state */
  return NULL;
}

/* __bool__ */

static int
EG_nonzero(PySEnumGeneric *self) {
  return self->nonzero;
}

static PyNumberMethods EG_as_number = {
    0,     /*nb_add*/
    0,     /*nb_subtract*/
    0,     /*nb_multiply*/
    0,     /*nb_divide*/
    0,     /*nb_remainder*/
    0,     /*nb_divmod*/
    0,     /*nb_power*/
    0,     /*nb_negative*/
    0,     /*nb_positive*/
    0,     /*nb_absolute*/
    (inquiry)EG_nonzero,     /*nb_nonzero*/
    0,    /*nb_invert*/
};

/* methods and attributes */

static PyObject *
EG_Reset(PySEnumGeneric *self, PyObject *args) {
  pysIEnumReset(self->enumerator);
  Py_INCREF(Py_None);
  return Py_None;
}

static PyMethodDef EG_methods[] = {
  {"Reset", (PyCFunction)EG_Reset, METH_NOARGS, NULL },
  {NULL, NULL}
};

PyTypeObject PySEnumGeneric_Type = {
  /* The ob_type field must be initialized in the module init function
   * to be portable to Windows without using C++. */
  PyObject_HEAD_INIT(NULL)
  0,          /*ob_size*/
  "_savi.EnumGeneric",     /*tp_name*/
  sizeof(PySEnumGeneric),   /*tp_basicsize*/
  0,          /*tp_itemsize*/
  /* methods */
  (destructor)EG_dealloc, /*tp_dealloc*/
  0,          /*tp_print*/
  0, /*tp_getattr*/
  0,          /*tp_setattr*/
  0,          /*tp_compare*/
  0,          /*tp_repr*/
  &EG_as_number, /*tp_as_number*/
  0,          /*tp_as_sequence*/
  0,          /*tp_as_mapping*/
  0,          /*tp_hash*/
  0,          /*tp_call*/
  0,          /*tp_str*/
  PyObject_GenericGetAttr,          /*tp_getattro*/
  0,          /*tp_setattro*/
  0,          /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT,          /*tp_flags*/
  0,          /*tp_doc*/
  0,          /*tp_traverse*/
  0,          /*tp_clear*/
  0,          /*tp_richcompare*/
  0,          /*tp_weaklistoffset*/
  (getiterfunc)EG_iter,      /*tp_iter*/
  (iternextfunc)EG_iternext, /*tp_iternext*/
  EG_methods,            /*tp_methods*/
  0,          /*tp_members*/
  0,          /*tp_getset*/
  0,          /*tp_base*/
};

/* vim:et:ts=2:noai:
 */
