/* $Id: pysaviStream.c,v 1.3 2005/03/11 03:20:49 mjp Exp $
 *
 * Adapted from 'savistrm.c' (Copyright  1999,2003, Sophos Plc)
 */

#include <Python.h>
#include <stdbool.h>
#include <string.h>     /* memcpy */
#include "csavi3c.h"
#include "pysaviCOM.h"  /* PYSCOM_* */
#include "pysaviUtil.h" /* pysPyObjHasMethod */

typedef struct _CIFileish_Struct
{
  CISaviStreamVtbl* pVtbl;
  SOPHOS_ULONG      refCount;
  IID               typeCode;
  PyObject *        f;
  int               flushOK;
  PyThreadState *   tstate;
} CIFileish;

static HRESULT      SOPHOS_STDCALL_PUBLIC CIFileish_QueryInterface(void* object, REFIID pIId, void** ppI);
static SOPHOS_ULONG SOPHOS_STDCALL_PUBLIC CIFileish_AddRef        (void* object);
static SOPHOS_ULONG SOPHOS_STDCALL_PUBLIC CIFileish_Release       (void* object);
static HRESULT      SOPHOS_STDCALL_PUBLIC CIFileish_ReadStream    (void* object, void* lpvBuffer, U32 uCount, U32* bytesRead);
static HRESULT      SOPHOS_STDCALL_PUBLIC CIFileish_WriteStream   (void* object, const void* lpvBuffer, U32 uCount, U32* bytesWritten);
static HRESULT      SOPHOS_STDCALL_PUBLIC CIFileish_SeekStream    (void* object, S32 lOffset, U32 uOrigin, U32* newPosition);
static HRESULT      SOPHOS_STDCALL_PUBLIC CIFileish_GetLength     (void* object, U32* length);

static const CISaviStreamVtbl CIFileishVtbl_Instance =
{
  CIFileish_QueryInterface,
  CIFileish_AddRef,
  CIFileish_Release,
  CIFileish_ReadStream,
  CIFileish_WriteStream,
  CIFileish_SeekStream,
  CIFileish_GetLength
};

/* CIFileish_tell(CIFileish, &POS): boolean
 *
 * args: CIFileish object ptr, U32 position output parameter
 * rtrn: true if successful, false otherwise
 * s-fx: Invokes object's tell() method, may raise Python exception
 * n.b.: caller MUST have threadstate/GIL
 */
static bool
CIFileish_tell(CIFileish *cif, U32 *pos) {
  PyObject *ret = NULL;

  DBG("CIFileish_tell(%p).tell()\n", cif);
  if (!(ret = PyObject_CallMethod(cif->f, "tell", "()")))
    return false;

  *pos = (U32) PyInt_AsLong(ret);
  Py_DECREF(ret);  

  if (*pos == -1 && PyErr_Occurred()) {
    /* Reset the error */
    PyErr_SetString(PyExc_TypeError,
		    "file-like tell() method must return a number");
	return false;
  }
  /* FIXME - broken tell() might return < 0.  Not good */

  DBG("CIFileish_tell(%p) returning...\n", cif);
  return true;
}

/* CIFileish_seek(CIFileish, OFFSET, WHENCE, &POS): boolean
 *
 * args: CIFileish object ptr, U32 offset, U32 whence, U32 position output
 *       parameter
 * rtrn: true if successful, false otherwise
 * s-fx: Invokes object's seek() and tell() methods, may raise Python
 *       exception
 * n.b.: caller MUST have threadstate/GIL
 */
static bool
CIFileish_seek(CIFileish *cif, S32 off, U32 whence, U32 *pos) {
  PyObject *ret = NULL;

  DBG("CIFileish_seek(%p->%p)\n", cif, cif->f);

  ret = PyObject_CallMethod(cif->f, "seek", "(ii)", off, whence);
  DBG("CIFileish_seek(%p)d OK!\n", cif);
  if (NULL == ret) return false;
  Py_DECREF(ret); /* Py_None */

  if (pos && !CIFileish_tell(cif, pos)) return false;

  return true;
}

/*
static void delete_CISaviStream(void* object);
 */

/* ----- */

void *
CIFileish_ctor(PyObject *f)
{
  CIFileish* INTobject = NULL;

  DBG("CIFileish_ctor(from %p)\n", f);
  INTobject = (CIFileish *)PyMem_Malloc(sizeof(CIFileish));
  if (!INTobject)
    return NULL;

  INTobject->pVtbl    = (CISaviStreamVtbl*) &CIFileishVtbl_Instance;
  INTobject->typeCode = SOPHOS_IID_SAVISTREAM;
  INTobject->refCount = 1;
  INTobject->f        = f;
  Py_INCREF(f);
  INTobject->flushOK  = pysPyObjHasMethod(f, "flush");

  INTobject->tstate = PyThreadState_Get();
  return (void *)INTobject;
}

static void
CIFileish_dtor(void *object) {
  CIFileish *INTobject = (CIFileish *)object;

  if (!object || !IsEqualIID(&INTobject->typeCode, &SOPHOS_IID_SAVISTREAM))
    return;

  Py_DECREF(INTobject->f);
  PyMem_Free(INTobject);
}

PYSCOM_QUERYINTERFACE(CIFileish, CIFileish, SOPHOS_IID_SAVISTREAM)
PYSCOM_ADDREF(CIFileish, CIFileish, SOPHOS_IID_SAVISTREAM)
PYSCOM_RELEASE(CIFileish, CIFileish, SOPHOS_IID_SAVISTREAM, CIFileish_dtor)

static HRESULT SOPHOS_STDCALL_PUBLIC CIFileish_ReadStream(void* object, void* lpvBuffer, U32 uCount, U32* bytesRead)
{
  CIFileish* INTobject = (CIFileish*)object;
  PyObject *ret = NULL;
  int howMany = -1;
  HRESULT hr;

  DBG("CIFileish_ReadStream(%p)\n", object);
  if (!object || !lpvBuffer || uCount == 0xFFFFFFFF || !bytesRead)
    return SOPHOS_E_INVALIDARG;

  PyEval_AcquireThread(INTobject->tstate);
  ret = PyObject_CallMethod(INTobject->f, "read", "(i)", uCount);
  if (NULL == ret) goto fail;
  if (!PyString_Check(ret)) {
    PyErr_SetString(PyExc_TypeError, "file-like object's read() method must return a byte string");
    goto fail;
  }
  howMany = PyString_Size(ret);
  memcpy(lpvBuffer, PyString_AsString(ret), howMany);
  Py_DECREF(ret);

  if (howMany>= 0) {
    *bytesRead = (U32)howMany;
    hr = SOPHOS_S_OK;
  } else {
    goto fail;  /* Should not occur */
  }

out:
  PyEval_ReleaseThread(INTobject->tstate);
  return hr;

fail:
  Py_XDECREF(ret);
  *bytesRead = 0;
  hr = SOPHOS_SAVI_ERROR_STREAM_READ_FAIL;
  goto out;
}

static HRESULT SOPHOS_STDCALL_PUBLIC CIFileish_WriteStream(void* object, const void* lpvBuffer, U32 uCount, U32* bytesWritten)
{
  CIFileish* INTobject = (CIFileish*)object;
  PyObject *ret = NULL;
  HRESULT hr = SOPHOS_SAVI_ERROR_STREAM_WRITE_FAIL;
  
  DBG("CIFileish_WriteStream(%p)\n", object);
  if (!object || !lpvBuffer || uCount == 0xFFFFFFFF || !bytesWritten)
    return SOPHOS_E_INVALIDARG;

  PyEval_AcquireThread(INTobject->tstate);
  ret = PyObject_CallMethod(INTobject->f, "write", "(s#)", lpvBuffer, uCount);
  if (ret && INTobject->flushOK) {
    Py_DECREF(ret); /* Py_None */
    ret = PyObject_CallMethod(INTobject->f, "flush", "()");
  }
  if (ret) {
    Py_DECREF(ret);
    *bytesWritten = (U32)uCount;
    hr = SOPHOS_S_OK;
  } else {
    goto failure;
  }

out:
  PyEval_ReleaseThread(INTobject->tstate);
  return hr;

failure:
  *bytesWritten = 0;
  hr = SOPHOS_SAVI_ERROR_STREAM_WRITE_FAIL;
  goto out;
}

static HRESULT SOPHOS_STDCALL_PUBLIC CIFileish_SeekStream(void* object, S32 lOffset, U32 uOrigin, U32* newPosition)
{
  CIFileish* INTobject = (CIFileish*)object;
  int                  whence    = -1;
  bool ok = false;

  DBG("CIFileish_SeekStream(%p)\n", object);
  if (!object || !newPosition)
    return SOPHOS_E_INVALIDARG;
  *newPosition = (U32)-1;
   
  if (uOrigin == SAVISTREAM_SEEK_SET)
    whence = SEEK_SET;
  else if (uOrigin == SAVISTREAM_SEEK_CUR)
    whence = SEEK_CUR;
  else if (uOrigin == SAVISTREAM_SEEK_END)
    whence = SEEK_END;
  else 
    return SOPHOS_E_INVALIDARG;

  PyEval_AcquireThread(INTobject->tstate);
  ok = CIFileish_seek(INTobject, lOffset, whence, newPosition);
  PyEval_ReleaseThread(INTobject->tstate);

  return ok ? SOPHOS_S_OK : SOPHOS_SAVI_ERROR_STREAM_SEEK_FAIL;
}

static HRESULT SOPHOS_STDCALL_PUBLIC CIFileish_GetLength(void* object, U32* length)
{
  CIFileish* INTobject = (CIFileish*)object;
  U32 curPos, endPos;

  DBG("CIFileish_GetLength(%p)\n", object);
  if (!object || !length)
    return SOPHOS_E_INVALIDARG;

  PyEval_AcquireThread(INTobject->tstate);
  if (!CIFileish_tell(INTobject, &curPos) ||
      !CIFileish_seek(INTobject, 0, SEEK_END, &endPos) ||
      !CIFileish_seek(INTobject, curPos, SEEK_SET, NULL)) {
    PyEval_ReleaseThread(INTobject->tstate);
    *length = 0;
    return SOPHOS_SAVI_ERROR_STREAM_GETLENGTH_FAIL;
  }
  PyEval_ReleaseThread(INTobject->tstate);

  *length = endPos;
  return SOPHOS_S_OK;
}
