#! /usr/bin/python

# $Id: test.py,v 1.3 2005/02/17 05:13:40 mjp Exp $

import unittest
import tempfile
import pysavi
import os
try:
    import cStringIO
    StringIO = cStringIO
except:
    import StringIO

MODTYPES = ['Savi3', 'SweepResults', 'SweepError', 'EnumGeneric',
            'IDEDetails', 'EngineConfig', 'VersionChecksum']

eicar = '*H+H$!ELIF-TSET-SURIVITNA-DRADNATS-RACIE$}7)CC7)^P(45XZP\\4[PA@%P!O5X'
eicar = reduce(lambda x,y: y+x, eicar)

checkpoints = 0

def isintegral(thing):
    return isinstance(thing, (int, long))

def hasmethod(thing, method):
    yesno = False
    try:
        yesno = callable(getattr(thing, method))
    except:
        pass
    return yesno

class BasicTestCases(unittest.TestCase):
    def test01Types(self):
        for sym in MODTYPES:
            t = pysavi.__dict__.get(sym)
            if t is None:
                self.fail("pysavi.%s undefined" % sym)
            self.failUnless(type(t) is type, "pysavi.%s not a type!")

            leftover = [ t for t in dir(pysavi) if type(t) is type ]
            leftover = [ t for t in leftover if t not in MODTYPES ]

            if leftover:
                self.fail("Untested types! " + str(tuple(leftover)))

    def test10TypeInheritance(self):
        '''
        Only Savi3 is sub-classable; other types are not.
        '''

        try:
            class Subby(pysavi.Savi3):
                pass
        except:
            self.fail("Savi3 not subclassable")

        types = [ getattr(pysavi, t) for t in MODTYPES if t != "Savi3" ]
        for t in types:
            try:
                class Subby(t):
                    pass
            except:
                pass
            else:
                self.fail(str(t) + " unexpectedly subclassable")

    def test20IterableTypes(self):
        eg = pysavi.EnumGeneric
        self.failUnless(hasmethod(eg, "__iter__"),
                                  "EnumGeneric lacks __iter__()")

    def test30Docstrings(self):
        need_doc = [ getattr(pysavi, t) for t in MODTYPES ]
        need_doc.append(pysavi)
        for nd in need_doc:
            self.failUnless(hasattr(nd, '__doc__'),
                            str(nd) + ' lacks __doc__')

    def test40Constants(self):
        # make sure at least a few are present
        consts = []
        for (sym, val) in pysavi.__dict__.iteritems():
            if not sym.startswith('SOPHOS_') or not isintegral(val):
                continue
            consts.append(sym)
        for sym in ('SOPHOS_S_OK', 'SOPHOS_TYPE_BOOLEAN', 'SOPHOS_TYPE_IDE',
                    'SOPHOS_ACTVTY_DECOMPR',
                    'SOPHOS_SAVI_ERROR_DISINFECTION_FAILED'):
            self.failUnless(sym in consts, "constant %s missing" % sym)

    def test50HelperFunctions(self):
        for f in ('SOPHOS_SUCCEEDED', 'SOPHOS_FAILED'):
            try:
                ret = getattr(pysavi, f)(pysavi.SOPHOS_S_OK)
                self.assert_(isinstance(ret, (int, long)), "Bummer")
            except:
                self.fail('%s() unexpected behavior' % f)

        try:
            ret = getattr(pysavi, 'strerror')(pysavi.SOPHOS_S_OK)
            self.assert_(isinstance(ret, str), "Bummer")
        except:
            self.fail('strerror() unexpected behavior')

class ScanningTestCases(unittest.TestCase):
    def __init__(self, *a, **kw):
        unittest.TestCase.__init__(self, *a, **kw)
        self.savi = pysavi.Savi3("A/ScanningTestCases") # may fail!
 
    def test10SweepBuffer(self): self.chkEicar('SweepBuffer', 'RACIE')

    def test20SweepFile(self):   self.chkEicar('SweepFile')

    def test30SweepHandle(self): self.chkEicar('SweepHandle', 'RACIE')

    def test40SweepStream(self): self.chkEicar('SweepStream', 'RACIE')

    def makeFile(self, buf):
        '''make a filename'''
        fn = tempfile.mktemp()
        file(fn, "w+").write(buf)
        return fn

    def destroyFile(self, name): os.unlink(name)

    def makeHandle(self, buf):
        fn = self.makeFile(buf)
        ret = file(fn, "r")
        os.unlink(fn)
        return ret

    def makeStream(self, buf): return StringIO.StringIO(buf)

    def chkEicar(self, meth, name = None):
        scan_what = eicar

        # Does the method to test exist?
        method = getattr(self.savi, meth)

        shortname = meth
        if meth.startswith('Sweep'): shortname = shortname[5:]
        # preparation work?
        pre = getattr(self, "make" + shortname, None)
        if pre is not None: scan_what = pre(eicar)

        if name is None:
            # SweepFile takes no 'name' argument
            v = method(scan_what)
            expected = (("EICAR-AV-Test", scan_what),)
        else:
            v = method(scan_what, name = name)
            expected = (("EICAR-AV-Test", "RACIE"),)
        try:
            self.validateSweepReturn(v, expected)
        except Exception, e:
            self.fail(name + "(eicar): " + str(e))

        # shutdown work?
        post = getattr(self, "destroy" + shortname, None)
        if post is not None: post(scan_what)

    def validateSweepReturn(self, eg, expected = None):
        '''
        Check the submitted SweepResults iterator for basic type integrity.
        If expected tuple is given, make sure that the VirusName and
        LocationInfo... attributes match.
        '''
        self.failUnless(isinstance(eg, pysavi.EnumGeneric),
                        "object is not type EnumGeneric (%s)" % str(eg))

        yesno = bool(eg)

        fetch1 = tuple([ (v.VirusName, v.LocationInformation) for v in eg ])

        if expected is not None:
            self.failUnless(fetch1 == expected,
                            "unexpected SweepResults: " + str(fetch1))

        self.failUnless(yesno == bool(fetch1),
                        "SweepResults __nonzero__ broken")

        self.failUnless(bool(eg) == bool(fetch1),
                        "SweepResults __nonzero__ broken after iteration")

        self.failUnless(eg.Reset() is None,
                        "SweepResults.Reset() unexpected behavior")

        self.failUnless(bool(eg) == bool(fetch1),
                        "SweepResults __nonzero__ broken after Reset")

        fetch2 = tuple([ (v.VirusName, v.LocationInformation) for v in eg ])

        if expected is not None:
            self.failUnless(fetch2 == expected,
                            "unexpected SweepResults after Reset")

        self.failUnless(fetch1 == fetch2,
                        "SweepResults differed after Reset")

def suite():
    basic = unittest.makeSuite(BasicTestCases, "test")
    scan  = unittest.makeSuite(ScanningTestCases, "test")
    return unittest.TestSuite((basic, scan))

def main():
    runner = unittest.TextTestRunner(verbosity = 3)
    runner.run(suite())

if __name__ == '__main__':
    main()

# vim:et:noai:ts=4:
